/*
 * main.c - main for Flash Magic ISP control
 *   by uratan! 2014.10.16
 */
//#define __MAIN_C__		/* <== inform common.h to alloc variables */
#include "common.h"
#include <chip.h>
	/*
	 * my LPC810 (DIP8)
	 *   +------------+
	 *   | LPC810     |
	 *   ) P  M021F   |
	 *   ) X  1M28603 |
	 *   | N  +D3414C | ... year 3, week 41, 4C' ver.13.4
	 *   +------------+
	 *      | Dev.ID:  0x00008100
	 *      | B.L.Ver: 13.4
	 */
/*
 * control boot for ISP/run of target by single RTS signal
 *
 * I/O pin usage
 *   pin#8: PIO0_0 (input)  - input (RTS)
 *   pin#3: PIO0_3 (output) - ISP (PIO0_1)
 *   pin#4: PIO0_2 (output) - RESET
 *
 * (1) rise-edge of RTS which width is near 150msec
 *     ==> boot ISP mode
 * (2) rise-edge of RTS which width is near 50msec
 *     ==> run target
 */
#define GPI_RTS    (1<<0)
#define GPO_ISP    (1<<3)
#define GPO_RESET  (1<<2)
#define GPO_Both  (GPO_ISP | GPO_RESET)
	/*
	 * make test pusle by uart (110bps)
	 *   n      width    bit-pattern     char
	 *  --------------------------------------------
	 *   1 ...  9.1msec  01010_10101 <-- 'U' (0x55)
	 *   2 ... 18.2msec  00100_11001 <-- '2' (0x32)
	 *   3 ... 27.3msec  01000_11001 <-- '1' (0x31)
	 *   4 ... 36.4msec  00001_11001 <-- '8' (0x38)
	 *   5 ... 45.5msec  00000_11001 <-- '0' (0x30)
	 *   6 ... 54.6msec  00000_01001 <-- SPC (0x20)
	 *   7 ... 63.7msec  01000_00001 <-- ^A  (0x01)
	 *   8 ... 72.8msec  00000_00011 <---(none) (0x80)
	 *   9 ... 81.9msec  00000_00001 <-- ^@  (0x00)
	 *  --------------------------------------------
	 *      (uart is LSB-first, with start/stop bit)
	 */

/*
 * do not disable pullUP-Rs of output port
 */
#define KEEP_PULL_UP   0

/*
 * SysTick base interval
 */
#define SYSTICK_Hz   1000		/* get 1msec interval */

/*
 * functions
 */
int main(void);
void configure_fixed_pins(void);
void configure_GPIO(void);
u_long initial_sysCLK(u_long divider);
void init_SysTick(u_long sysCLK);
void SysTick_Handler(void)        ATTR_INTR;
void def_Exception_Handler(void)  ATTR_INTR;
void def_NMI_Handler(void)        ATTR_INTR;
void def_HardFault_Handler(void)  ATTR_INTR;
void def_SVCall_Handler(void)     ATTR_INTR;
void def_PendSV_Handler(void)     ATTR_INTR;

/*
 * variables
 */
VOLATILE int systick_raw;


/* =============================================================== */

/*
 *
 */
int main(void)
{
	u_long sysCLK;
	int low_counter;
	int isp_state;

	/*
	 * mask all interrupts by PRIMASK flag in the core
	 */
	__disable_irq();

	/*
	 * initial my variables
	 */
	systick_raw = 0;

	/*
	 * configure I/O pins
	 */
	configure_fixed_pins();
	configure_GPIO();

	/*
	 * setup system speed (1.2MHz is enough)
	 */
	sysCLK = initial_sysCLK(/*divider=*/ 10);

	/*
	 * initial peripherals
	 */
	init_SysTick(sysCLK);

	/*
	 * start my job
	 */
	low_counter = 0;
	isp_state = 0;

	__enable_irq();

	while(1) {
		/*
		 * wait for systick
		 */
		if(!systick_raw) {
			continue;
		}

		/*
		 * get systick, interval is 1msec
		 */
		systick_raw = 0;

		/*
		 * check RTS signal level
		 */
		if(LPC_GPIO_PORT->PIN[0] & GPI_RTS) {
			/*
			 * RTS is high, examine pulse width
			 */
			if((145 <= low_counter) && (low_counter <= 180)) {
				/*
				 * 150msec low pulse is detected
				 * make target in ISP mode
				 */
				isp_state = 1;
			}
			else if((45 <= low_counter) && (low_counter <= 75)) {
				/*
				 * 50msec low pulse is detected
				 * run target
				 */
				isp_state = -1;
			}

			/*
			 * reset the counter for next
			 */
			low_counter = 0;
		} else {
			/*
			 * RTS is low, count low length
			 */
			if(low_counter < 1000) {
				low_counter++;
			}
		}

		/*
		 * control ISP/RESET signals by every systick (1msec)
		 */
		switch(isp_state) {
			/*
			 * not to driver any at default state
			 */
		case 0:
		default:
				LPC_GPIO_PORT->DIR[0] &= ~GPO_Both;		/* release both */
				isp_state = 0;
				break;

			/*
			 * positive states to boot ISP
			 */
		case 1:
				LPC_GPIO_PORT->DIR[0] |= GPO_Both;		/* sink both */
				isp_state = 2;
				break;
		case 2:
				LPC_GPIO_PORT->DIR[0] &= ~GPO_RESET;	/* release RESET */
				isp_state = 3;
				break;
		case 3:
				LPC_GPIO_PORT->DIR[0] &= ~GPO_ISP;		/* release ISP */
				isp_state = 0;
				break;

			/*
			 * negative states to run the target
			 */
		case -1:
				LPC_GPIO_PORT->DIR[0] &= ~GPO_ISP;		/* release ISP */
				LPC_GPIO_PORT->DIR[0] |= GPO_RESET;		/* sink RESET */
				isp_state = -2;
				break;
		case -2:
				LPC_GPIO_PORT->DIR[0] &= ~GPO_RESET;	/* release RESET */
				isp_state = 0;
				break;
		}
	}

	/* never return */
}

/* =============================================================== */

/*
 * determine how handle fixed pins
 */
void configure_fixed_pins(void)
{
	u_long tmp;

	tmp = LPC_SWM->PINENABLE0;

	/*
	 * disable these fixed-pin-function, GPIO will have control
	 */
	tmp |= 0
			| (1<<2)		/* disable SWCLK, PIO0_3 enabled */
			| (1<<3)		/* disable SWDIO, PIO0_2 enabled */
		//  | (1<<6)		/* disable RESET, PIO0_5 enabled */
			| 0;

	/*
	 * enable these fixed-pin-function, GPIO will lose control
	 *   (do not enable both ACMP_I2 and CLKIN at same time)
	 */
	tmp &= ~(0
		//  | (1<<0)		/* enable ACMP_I1, PIO0_0 disabled */
		//  | (1<<1)		/* enable ACMP_I2, PIO0_1 disabled */
//NO-PIN//  | (1<<4)		/* enable XTALIN,  PIO0_8 disabled */
//NO-PIN//  | (1<<5)		/* enable XTALOUT, PIO0_9 disabled */
		//  | (1<<7)		/* enable CLKIN,   PIO0_1 disabled */
//NO-PIN//  | (1<<8)		/* enable VDDCMP,  PIO0_6 disabled */
			| 0);

	/*
	 * write back
	 */
	LPC_SWM->PINENABLE0 = tmp;
}

/*
 *
 */
void configure_GPIO(void)
{
	u_long n, tmp;

	/*
	 * prevent the the open-drain pins from internally floating
	 *   (PIO0_10 and PIO0_11, section 6.3, UM10601)
	 */
#define INTERNAL_OPEN_DRAIN_PINS   ((1<<10) | (1<<11))
	LPC_GPIO_PORT->CLR[0]  = INTERNAL_OPEN_DRAIN_PINS;	/* both Low */
	LPC_GPIO_PORT->DIR[0] |= INTERNAL_OPEN_DRAIN_PINS;	/* start to drive */

	/*
	 * setup GPIO
	 */
	LPC_GPIO_PORT->CLR[0] = GPO_ISP | GPO_RESET;	/* set Low */
	//LPC_GPIO_PORT->DIR[0] |= GPO_ISP | GPO_RESET;	/* start to drive */
		/*
		 * use these OUT port as open-drain
		 *   (ouch! he has OpenDrain mode !)
		 */

#if KEEP_PULL_UP
#else /* KEEP_PULL_UP */
	/*
	 * finally, disable pull-up-R on PIO0_2, PIO0_3
	 */
	LPC_SYSCTL->SYSAHBCLKCTRL |= (1 << SYSCTL_CLOCK_IOCON);

	n = IOCON_PIO2;
	tmp = LPC_IOCON->PIO0[n];
	tmp &= ~PIN_MODE_MASK;
	tmp |= (PIN_MODE_INACTIVE <<PIN_MODE_BITNUM);
	LPC_IOCON->PIO0[n] = tmp;

	n = IOCON_PIO3;
	tmp = LPC_IOCON->PIO0[n];
	tmp &= ~PIN_MODE_MASK;
	tmp |= (PIN_MODE_INACTIVE <<PIN_MODE_BITNUM);
	LPC_IOCON->PIO0[n] = tmp;

	LPC_SYSCTL->SYSAHBCLKCTRL &= ~(1 << SYSCTL_CLOCK_IOCON);
#endif /* KEEP_PULL_UP */
}

/* =============================================================== */

/*
 * configure system clock
 */
u_long initial_sysCLK(u_long divider)
{
	/*
	 * expect these conditions by cold start
	 */
//	LPC_SYSCTL->SYSAHBCLKDIV = 1;		/* 1/1 (reset value) */
//	LPC_SYSCTL->SYSPLLCLKSEL = 0;		/* sel IRC (reset value) */
//	LPC_SYSCTL->SYSPLLCLKUEN = 0;
//	LPC_SYSCTL->SYSPLLCLKUEN = 1;
//	LPC_SYSCTL->MAINCLKSEL = 0x0;		/* run with IRC (reset value) */
//	LPC_SYSCTL->MAINCLKUEN = 0;
//	LPC_SYSCTL->MAINCLKUEN = 1;

	LPC_SYSCTL->SYSAHBCLKDIV = divider;
	return(12*1000*1000 / divider);		/* return sysCLK */
}

/* =============================================================== */

/*
 * run SysTick timer
 */
void init_SysTick(u_long sysCLK)
{
	/*
	 * setup SysTick timer
	 */
	SysTick->LOAD = sysCLK / SYSTICK_Hz / 2 - 1;	/* use half-clock */
	SysTick->VAL = 0;			/* clear the counter (value is ignored) */
		/*
		 * 30,000,000 = 0x01C9_C380  >  (1<<24)
		 * 16,777,215 = 0x00FF_FFFF  =  (1<<24)
		 * 15,000,000 = 0x00E4_E1C0  <  (1<<24) ... half-clock covers 1Hz
		 */
	/*
	 * setup NVIC
	 */
	NVIC_SetPriority(SysTick_IRQn, PRI_SYSTICK);

	/*
	 * enable intr and run it
	 */
	SysTick->CTRL = 0x03;		/* run it with half-clock, intr enabled */
}

/*
 * interrupt service routine
 */
void SysTick_Handler(void)
{
	systick_raw = 1;
		/*
		 * interrupt request flag may be cleared automatically
		 */
}

/* =============================================================== */

/*
 * try to catch exceptions
 */

void def_Exception_Handler(void)
{
	while(1)
		;
	/* never return */
}

void def_NMI_Handler(void)
{
	while(1)
		;
	/* never return */
}

void def_HardFault_Handler(void)
{
	while(1)
		;
	/* never return */
}

void def_SVCall_Handler(void)
{
	while(1)
		;
	/* never return */
}

void def_PendSV_Handler(void)
{
	while(1)
		;
	/* never return */
}

/* =============================================================== */
